package fr.soleil.docking.vl;

import java.awt.Color;
import java.beans.PropertyChangeEvent;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

import javax.swing.JComponent;
import javax.swing.JOptionPane;
import javax.swing.UIManager;

import com.vldocking.swing.docking.DockingDesktop;
import com.vldocking.swing.docking.DockingPreferences;

import fr.soleil.docking.ADockingManager;
import fr.soleil.docking.exception.DockingException;
import fr.soleil.docking.perspective.IPerspective;
import fr.soleil.docking.perspective.PerspectiveFactory;
import fr.soleil.docking.vl.view.VlDockView;
import fr.soleil.docking.vl.view.VlDockViewFactory;

public class VlDockDockingManager extends ADockingManager {
	protected SoleilDockingDesktop mainDockingDesktop;

	public VlDockDockingManager() {
		super(new VlDockViewFactory(), new PerspectiveFactory());
		DockingPreferences.setShadowDesktopStyle();
		UIManager.getDefaults().put("TabbedDockableContainer.tabPlacement", 1);
	}

	@Override
	public JComponent getDockingArea() {
		if (mainDockingDesktop == null) {
			mainDockingDesktop = (SoleilDockingDesktop) new SoleilDockingDesktop();
			mainDockingDesktop.setOpaqueContents(true);
		}
		return mainDockingDesktop;
	}

	public DockingDesktop getDockingDesktop() {
		return (DockingDesktop) getDockingArea();
	}

	@Override
	public JComponent createNewDockingArea(Color background) {
		SoleilDockingDesktop result = new SoleilDockingDesktop(false, false,
				true, false);
		if (background != null) {
			result.setBackground(background);
		}
		return result;
	}

	@Override
	public void setDockingAreaBeackground(JComponent dockingArea,
			Color areaBackground) {
		if (dockingArea != null) {
			dockingArea.setBackground(areaBackground);
		}
	}

	@Override
	public void loadPerspective(IPerspective perspective)
			throws DockingException {
		DockingException dockingException = null;
		if (perspective.getByteArray().length > 0) {

			ObjectInputStream ois = null;
			ByteArrayInputStream bais = null;
			try {

				bais = new ByteArrayInputStream(perspective.getByteArray());
				ois = new ObjectInputStream(bais);
				mainDockingDesktop.readXML(ois);
			} catch (Exception e) {
				dockingException = new DockingException("Perspective.load() :"
						+ " Unexpected Error", e);
			} finally {
				try {
					if (ois != null) {
						ois.close();
					}
					// Actually there is no effect to close bais ...
				} catch (IOException e) {
					if (dockingException == null) {
						dockingException = new DockingException(
								"I/O Exception", e);
					} else {
						e.printStackTrace();
					}
				}
			}
		}
		if (dockingException != null) {
			throw dockingException;
		}
	}

	@Override
	protected void savePerspective(IPerspective perspective) {
		ByteArrayOutputStream baos = null;
		ObjectOutputStream ous = null;
		baos = new ByteArrayOutputStream();
		try {
			ous = new ObjectOutputStream(baos);
			mainDockingDesktop.writeXML(ous);
		} catch (IOException e) {
			e.printStackTrace();
			JOptionPane.showMessageDialog(mainDockingDesktop,
					"Perspective.save() :" + " Unexpected Error (see traces)",
					"VLDock - Error", JOptionPane.ERROR_MESSAGE);
		} finally {
			try {
				ous.close();
				// Actually there is no effect to close baos ...
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		perspective.setByteArray(baos.toByteArray());

	}

	@Override
	protected void updateViews(PropertyChangeEvent evt) {
		if (evt.getSource() == this.viewFactory) {
			VlDockView newView = (VlDockView) evt.getNewValue();
			VlDockView oldView = (VlDockView) evt.getOldValue();
			if (newView == null) { // removed

				mainDockingDesktop.remove(oldView);
			} else {
				mainDockingDesktop.addDockable(newView);
			}
		}
	}

	@Override
	public void setEnabledDocking(boolean enabledDocking) {
		System.out.println("VlDockDockingManager.setEnabledDocking()");
	}

	@Override
	public boolean isEnabledDocking() {
		return true;
	}
}
